from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model
from django.contrib.auth.models import User

UserModel = get_user_model()


class UsernameBackend(ModelBackend):
    def authenticate(self, username=None, password=None):
        """ Authenticate a user based on email address as the user name. """
        try:
            user = User.objects.get(username=username)
            if user.check_password(password):
                return user
        except User.DoesNotExist:
            return None

    def get_user(self, user_id):
        """ Get a User object from the user_id. """
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



class DisableCSRFMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        setattr(request, '_dont_enforce_csrf_checks', True)
        response = self.get_response(request)
        return response


from django.utils import timezone
from django.db import models


class Base(models.Model):  # base class should subclass 'django.db.models.Model'
    STATUS_CHOICES = (
        ('draft', 'Draft'),
        ('published', 'Published'),
    )
    slug = models.SlugField(max_length=250,
                            db_index=True,
                            unique=True,)
    publish = models.DateTimeField(default=timezone.now)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    status = models.CharField(max_length=10,
                              db_index=True,
                              choices=STATUS_CHOICES,
                              default='draft')

    class Meta:
        abstract = True  # Set this model as Abstract
        ordering = ('slug',)

        def __str__(self):
            return self.slug

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
"""
Custom serializer fields for nested translations.
"""
from __future__ import unicode_literals
from collections import OrderedDict
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.fields import SkipField
from parler.models import TranslatableModelMixin, TranslatedFieldsModel
from parler.utils.context import switch_language
from orion_common.orion_core.dependencies.parler_rest.utils import create_translated_fields_serializer


class TranslatedFieldsField(serializers.Field):
    """
    Exposing all translated fields for a TranslatableModel in REST style.
    """
    default_error_messages = dict(serializers.Field.default_error_messages, **{
        'invalid': _("Input is not a valid dict"),
        'empty': _("This field may not be empty.")
    })

    def __init__(self, *args, **kwargs):
        """
        Receive custom serializer class and model.
        """
        self.serializer_class = kwargs.pop('serializer_class', None)
        self.shared_model = kwargs.pop('shared_model', None)

        self.allow_empty = kwargs.pop('allow_empty', False)
        super(TranslatedFieldsField, self).__init__(*args, **kwargs)

    def bind(self, field_name, parent):
        """
        Create translation serializer dynamically.

        Takes translatable model class (shared_model) from parent serializer and it
        may create a serializer class on the fly if no custom class was specified.
        """
        super(TranslatedFieldsField, self).bind(field_name, parent)

        # Expect 1-on-1 for now. Allow using source as alias,
        # but it should not be a dotted path for now
        related_name = self.source or field_name

        # This could all be done in __init__(), but by moving the code here,
        # it's possible to auto-detect the parent model.
        if self.shared_model is not None and self.serializer_class is not None:
            return

        # Fill in the blanks
        if self.serializer_class is None:
            if self.shared_model is None:
                # Auto detect parent model
                from .serializers import TranslatableModelSerializerMixin
                if not isinstance(parent, TranslatableModelSerializerMixin):
                    raise TypeError("Expected 'TranslatableModelSerializer' as serializer base class")
                if not issubclass(parent.Meta.model, TranslatableModelMixin):
                    raise TypeError("Expected 'TranslatableModel' for the parent model")

                self.shared_model = parent.Meta.model

            # Create serializer based on shared model.
            translated_model = self.shared_model._parler_meta[related_name]
            self.serializer_class = create_translated_fields_serializer(
                self.shared_model, related_name=related_name,
                meta={'fields': translated_model.get_translated_fields()}
            )
        else:
            if not issubclass(self.serializer_class.Meta.model, TranslatedFieldsModel):
                raise TypeError("Expected 'TranslatedFieldsModel' for the serializer model")

            # On Django 1.8+ this works:
            #translated_fields_model = self.serializer_class.Meta.model
            #self.shared_model = translated_fields_model.master.field.related.model

    def to_representation(self, value):
        """
        Serialize translated fields.

        Simply iterate over available translations and, for each language,
        delegate serialization logic to the translation model serializer.

        Output languages can be selected by passing a list of language codes,
        `languages`, within the serialization context.
        """
        if value is None:
            return

        # Only need one serializer to create the native objects
        serializer = self.serializer_class(
            instance=self.parent.instance,  # Typically None
            context=self.context,
            partial=self.parent.partial
        )

        # Don't need to have a 'language_code', it will be split up already,
        # so this should avoid redundant output.
        if 'language_code' in serializer.fields:
            raise ImproperlyConfigured("Serializer may not have a 'language_code' field")

        translations = value.all()  # value = translations related manager
        languages = self.context.get('languages')
        if languages:
            translations = translations.filter(language_code__in=languages)

        # Split into a dictionary per language
        result = OrderedDict()
        for translation in translations:
            result[translation.language_code] = serializer.to_representation(translation)

        return result

    def to_internal_value(self, data):
        """
        Deserialize data from translations fields.

        For each received language, delegate validation logic to
        the translation model serializer.
        """
        if data is None:
            return

        if not isinstance(data, dict):
            self.fail('invalid')
        if not self.allow_empty and len(data) == 0:
            self.fail('empty')

        result, errors = {}, {}
        for lang_code, model_fields in data.items():
            serializer = self.serializer_class(data=model_fields)
            if serializer.is_valid():
                result[lang_code] = serializer.validated_data
            else:
                errors[lang_code] = serializer.errors

        if errors:
            raise serializers.ValidationError(errors)
        return result


class TranslatedField(serializers.Field):
    """
    Read-only field to expose a single object property in all it's languages.
    """

    def get_attribute(self, instance):
        # Instead of fetching the attribute with getattr() (that proxies to the Parler TranslatableField),
        # read the translation model directly to fetch all languages, and combine that into a dict.
        model = instance._parler_meta.get_model_by_field(self.source)  # This already validates the fields existance
        extension = instance._parler_meta[model]
        translations = getattr(instance, extension.rel_name)

        # Split into a dictionary per language
        value = OrderedDict()
        for translation in translations.all():  # Allow prefetch_related() to do it's work
            value[translation.language_code] = getattr(translation, self.source)
        return value

    def to_representation(self, value):
        return value

    # pylint: disable=no-self-use
    def to_internal_value(self, data):
        return data


class TranslatedAbsoluteUrlField(serializers.ReadOnlyField):
    """
    Allow adding an absolute URL to a given translation.
    """
    def get_attribute(self, instance):
        # When handling the create() all, skip this field.
        if isinstance(instance, (dict, OrderedDict)):
            raise SkipField()

        assert isinstance(instance, TranslatedFieldsModel), (
            ("The TranslatedAbsoluteUrlField can only be used on a TranslatableModelSerializer, "
             " not on a {0}".format(instance.__class__))
        )

        return instance

    def to_representation(self, value):
        request = self.context['request']
        with switch_language(value.master, value.language_code):
            return request.build_absolute_uri(value.master.get_absolute_url())



# -*- coding: utf-8 -*-
"""
Custom serializers suitable to translated models.
"""
from __future__ import absolute_import, unicode_literals
from rest_framework import serializers
from orion_common.orion_core.dependencies.parler_rest.fields import TranslatedField, TranslatedFieldsField


class TranslatableModelSerializerMixin(object):
    """
    Mixin class to be added to a :class:`rest_framework.serializers.ModelSerializer` .
    """

    def save(self, **kwargs):
        """
        Extract the translations and save them after main object save.

        By default all translations will be saved no matter if creating
        or updating an object. Users with more complex needs might define
        their own save and handle translation saving themselves.
        """
        translated_data = self._pop_translated_data()
        instance = super(TranslatableModelSerializerMixin, self).save(**kwargs)
        self.save_translations(instance, translated_data)
        return instance

    def _pop_translated_data(self):
        """
        Separate data of translated fields from other data.
        """
        translated_data = {}
        for field_name, field in self.get_fields().items():
            if isinstance(field, (TranslatedField, TranslatedFieldsField)):
                key = field.source or field_name
                translations = self.validated_data.pop(key, None)
                if translations:
                    translated_data[key] = translations
        return translated_data

    def save_translations(self, instance, translated_data):
        """
        Save translation data into translation objects.
        """
        for meta in self.Meta.model._parler_meta:
            for field_name, translations in translated_data.items():
                for lang_code, translation in translations.items():
                    model_field = instance._get_translated_model(lang_code, auto_create=True, meta=meta)
                    if meta.rel_name == field_name:
                        for trans_field, value in translation.items():
                            setattr(model_field, trans_field, value)
                    elif field_name in meta.get_translated_fields():
                        setattr(model_field, field_name, translation)

        # Go through the same hooks as the regular model,
        # instead of calling translation.save() directly.
        instance.save_translations()


class TranslatableModelSerializer(TranslatableModelSerializerMixin, serializers.ModelSerializer):
    """
    Serializer that saves :class:`TranslatedFieldsField` automatically.
    """
    pass



# -*- coding: utf-8 -*-
"""
Various utilities to ease integration with Rest Framework.
"""
from rest_framework import serializers


def create_translated_fields_serializer(shared_model, meta=None, related_name=None, **fields):
    """
    Create a Rest Framework serializer class for a translated fields model.

    :param shared_model: The shared model.
    :type shared_model: :class:`parler.models.TranslatableModel`
    """
    if not related_name:
        translated_model = shared_model._parler_meta.root_model
    else:
        translated_model = shared_model._parler_meta[related_name].model

    # Define inner Meta class
    if not meta:
        meta = {}
    meta['model'] = translated_model
    meta.setdefault('fields', ['language_code'] + translated_model.get_translated_fields())

    # Define serialize class attributes
    attrs = {}
    attrs.update(fields)
    attrs['Meta'] = type('Meta', (), meta)

    # Dynamically create the serializer class
    return type('{0}Serializer'.format(translated_model.__name__), (serializers.ModelSerializer,), attrs)



from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


def password_strength_validate(value):
    """Validates that a password is as least 7 characters long and has at least
       1 digit and 1 letter.
       """
    min_length = 7

    if len(value) < min_length:
        raise ValidationError(_('Password must be at least {0} characters '
                                'long.').format(min_length))

    # check for digit
    if not any(char.isdigit() for char in value):
        raise ValidationError(_('Password must contain at least 1 digit.'))

    # check for letter
    if not any(char.isalpha() for char in value):
        raise ValidationError(_('Password must contain at least 1 letter.'))



from rest_framework import permissions


class ActivePermission(permissions.BasePermission):
    def has_permission(self, request, view):
        if request.user.is_authenticated:
            return True
        else:
            return False




"""
Provides various authentication policies.
"""
import base64
import binascii

from django.contrib.auth import authenticate, get_user_model
from django.middleware.csrf import CsrfViewMiddleware
from django.utils.translation import gettext_lazy as _

from rest_framework import HTTP_HEADER_ENCODING, exceptions


def get_authorization_header(request):
    """
    Return request's 'Authorization:' header, as a bytestring.

    Hide some test client ickyness where the header can be unicode.
    """
    auth = request.META.get('HTTP_AUTHORIZATION', b'')
    if isinstance(auth, str):
        # Work around django test client oddness
        auth = auth.encode(HTTP_HEADER_ENCODING)
    return auth


class CSRFCheck(CsrfViewMiddleware):
    def _reject(self, request, reason):
        # Return the failure reason instead of an HttpResponse
        return reason


class BaseAuthentication:
    """
    All authentication classes should extend BaseAuthentication.
    """

    def authenticate(self, request):
        """
        Authenticate the request and return a two-tuple of (user, token).
        """
        raise NotImplementedError(".authenticate() must be overridden.")

    def authenticate_header(self, request):
        """
        Return a string to be used as the value of the `WWW-Authenticate`
        header in a `401 Unauthenticated` response, or `None` if the
        authentication scheme should return `403 Permission Denied` responses.
        """
        pass


class BasicAuthentication(BaseAuthentication):
    """
    HTTP Basic authentication against username/password.
    """
    www_authenticate_realm = 'api'

    def authenticate(self, request):
        """
        Returns a `User` if a correct username and password have been supplied
        using HTTP Basic authentication.  Otherwise returns `None`.
        """
        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != b'basic':
            return None

        if len(auth) == 1:
            msg = _('Invalid basic header. No credentials provided.')
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _('Invalid basic header. Credentials string should not contain spaces.')
            raise exceptions.AuthenticationFailed(msg)

        try:
            try:
                auth_decoded = base64.b64decode(auth[1]).decode('utf-8')
            except UnicodeDecodeError:
                auth_decoded = base64.b64decode(auth[1]).decode('latin-1')
            auth_parts = auth_decoded.partition(':')
        except (TypeError, UnicodeDecodeError, binascii.Error):
            msg = _('Invalid basic header. Credentials not correctly base64 encoded.')
            raise exceptions.AuthenticationFailed(msg)

        userid, password = auth_parts[0], auth_parts[2]
        return self.authenticate_credentials(userid, password, request)

    def authenticate_credentials(self, userid, password, request=None):
        """
        Authenticate the userid and password against username and password
        with optional request for context.
        """
        credentials = {
            get_user_model().USERNAME_FIELD: userid,
            'password': password
        }
        user = authenticate(request=request, **credentials)

        if user is None:
            raise exceptions.AuthenticationFailed(_('Invalid username/password.'))

        if not user.is_active:
            raise exceptions.AuthenticationFailed(_('User inactive or deleted.'))

        return (user, None)

    def authenticate_header(self, request):
        return 'Basic realm="%s"' % self.www_authenticate_realm


class SessionAuthentication(BaseAuthentication):
    """
    Use Django's session framework for authentication.
    """

    def authenticate(self, request):
        """
        Returns a `User` if the request session currently has a logged in user.
        Otherwise returns `None`.
        """

        # Get the session-based user from the underlying HttpRequest object
        user = getattr(request._request, 'user', None)

        # Unauthenticated, CSRF validation not required
        if not user or not user.is_active:
            return None

        self.enforce_csrf(request)

        # CSRF passed with authenticated user
        return (user, None)

    def enforce_csrf(self, request):
        """
        Enforce CSRF validation for session based authentication.
        """
        def dummy_get_response(request):  # pragma: no cover
            return None

        check = CSRFCheck(dummy_get_response)
        # populates request.META['CSRF_COOKIE'], which is used in process_view()
        check.process_request(request)
        reason = check.process_view(request, None, (), {})
        if reason:
            # CSRF failed, bail with explicit error message
            raise exceptions.PermissionDenied('CSRF Failed: %s' % reason)


class TokenAuthentication(BaseAuthentication):
    """
    Simple token based authentication.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string "Token ".  For example:

        Authorization: Token 401f7ac837da42b97f613d789819ff93537bee6a
    """

    keyword = 'Token'
    model = None

    def get_model(self):
        if self.model is not None:
            return self.model
        from rest_framework.authtoken.models import Token
        return Token

    """
    A custom token model may be used, but must have the following properties.

    * key -- The string identifying the token
    * user -- The user to which the token belongs
    """

    def authenticate(self, request):
        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != self.keyword.lower().encode():
            return None

        if len(auth) == 1:
            msg = _('Invalid token header. No credentials provided.')
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _('Invalid token header. Token string should not contain spaces.')
            raise exceptions.AuthenticationFailed(msg)

        try:
            token = auth[1].decode()
        except UnicodeError:
            msg = _('Invalid token header. Token string should not contain invalid characters.')
            raise exceptions.AuthenticationFailed(msg)

        return self.authenticate_credentials(token)

    def authenticate_credentials(self, key):
        model = self.get_model()
        try:
            token = model.objects.select_related('user').get(key=key)
        except model.DoesNotExist:
            raise exceptions.AuthenticationFailed(_('Invalid token.'))

        if not token.user.is_active:
            raise exceptions.AuthenticationFailed(_('User inactive or deleted.'))

        return (token.user, token)

    def authenticate_header(self, request):
        return self.keyword


class RemoteUserAuthentication(BaseAuthentication):
    """
    REMOTE_USER authentication.

    To use this, set up your web server to perform authentication, which will
    set the REMOTE_USER environment variable. You will need to have
    'django.contrib.auth.backends.RemoteUserBackend in your
    AUTHENTICATION_BACKENDS setting
    """

    # Name of request header to grab username from.  This will be the key as
    # used in the request.META dictionary, i.e. the normalization of headers to
    # all uppercase and the addition of "HTTP_" prefix apply.
    header = "REMOTE_USER"

    def authenticate(self, request):
        user = authenticate(request=request, remote_user=request.META.get(self.header))
        if user and user.is_active:
            return (user, None)



# coding=utf-8
def saludar():
    print("Hola, te estoy saludando desde la función saludar() del módulo saludos")


class Saludo:
    def __init__(self):
        print("Hola, te estoy saludando desde el __init__ de la clase Saludo")


def despedir():
    print("Adiós, me estoy despidiendo desde la función despedir() del módulo despedidas")


class Despedida:
    def __init__(self):
        print("Adiós, me estoy despidiendo desde el __init__ de la clase Despedida")



from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class Pagination(PageNumberPagination):
    page_size = 5
    max_page_size = 1000
    page_size_query_param = 'limit'

    # def get_paginated_response(self, data):
    #     return Response({
    #         'count': self.page.paginator.count,
    #         'num_pages': self.page.paginator.num_pages,
    #         'page_number': self.page.number,
    #         'page_size': self.page_size,
    #         'next_link': self.get_next_link(),
    #         'previous_link': self.get_previous_link(),
    #         'results': data
    #     })



import subprocess
from django.apps import apps

print(f'______________________begin pwd______________________')
print(subprocess.call(['pwd']))
print(f'______________________end pwd______________________')

subprocess.call(['rm', '-rf', 'fixtures'])
subprocess.call(['mkdir', 'fixtures'])

for model in apps.get_models():
    model_str = f'{model._meta.app_label}.{model.__name__}'
    print(f'{model_str}')
    subprocess.call(['mkdir', f'fixtures/{model_str}'])
    print(f'/fixtures/{model_str}')
    print(f'python', 'manage.py', 'dumpdata', '--indent', '2', '--output', f'fixtures/{model_str}/{model_str}.json')
    subprocess.call(['python',
                     'manage.py', 'dumpdata',
                     '--indent', '2',
                     '--output', f'fixtures/{model_str}/{model_str}.json'])





from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.admin.views.decorators import user_passes_test


def superuser_required(view_func=None,
                       redirect_field_name=REDIRECT_FIELD_NAME,
                       login_url='account_login_url'):
    """
    Decorator for views that checks that the user is logged in and is a
    superuser, redirecting to the login page if necessary.
    """
    actual_decorator = user_passes_test(
        lambda u: u.is_active and u.is_superuser,
        login_url=login_url,
        redirect_field_name=redirect_field_name
    )
    if view_func:
        return actual_decorator(view_func)
    return actual_decorator



from rest_framework import permissions


class SuperPermission(permissions.BasePermission):
    def has_permission(self, request, view):
        if request.method in permissions.SAFE_METHODS:
            return bool(request.user.is_superuser and request.user.profile.active)
        else:
            return bool(request.user.is_superuser and request.user.profile.active)



from parler.admin import TranslatableAdmin


class BaseAdmin(TranslatableAdmin):  # base class should subclass 'django.db.models.Model'
    list_display = ['id',
                    'slug',
                    'publish',
                    'created',
                    'updated',
                    'status']
    list_filter = ['id',
                   'slug',
                   'publish',
                   'created',
                   'updated',
                   'status']
    list_editable = ['status']

    class Meta:
        abstract = True  # Set this model as Abstract
        ordering = ('slug',)

        def __str__(self):
            return self.slug

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



from django.http import HttpResponse
from rest_framework.renderers import JSONRenderer


class JsonResponse(HttpResponse):
    def __init__(self, data, **kwargs):
        content = JSONRenderer().render(data)
        kwargs['content_type'] = 'application/json'
        super(JsonResponse, self).__init__(content, **kwargs)